/***************************************************************************************/
/* Demo for driving PIC16Fxx as I2C-Slave according to Application Note AN734          */
/*                                                                                     */
/* Slave                                                                               */
/*                                                                                     */
/* Description: This example is indended to demonstrate several capabilities of I2C bus*/
/*              - master/slave communication                                           */
/*              - timeout in case of communiation failure                              */
/*              - capability of sending unsigned char variables to 32 different        */
/*                internal UC registers                                                */
/*              - variable data lenght                                                 */
/*                                                                                     */
/* Mikrocontroller:       PIC16F887                                                    */
/* Clock:                 internal clock (8.0 MHz)                                     */
/*                                                                                     */
/* Compiler:              mikroC V6.6.2                                                */
/*                                                                                     */
/* Developer:            Mitroi Nicolae Alexandru                                      */
/* Stert of software development: 01. August 2016                                      */
/* Last updated:                  05. August 2016                                      */
/***************************************************************************************/


/******************************************************************************
#definitions section
******************************************************************************/
 #define _I2C_MAX_REGISTERS_COUNT  32
 

/***************************************************************************************/
/* #include section                                                                    */
/***************************************************************************************/
 #include <PIC16F_I2C_SlaveFunctions.h>


/****************** Interrupt-Service-Routine (ISR) ************************************/

/***************************************************************************************/
/* Interrupt Service Routine:                                                          */
/*                                                                                     */
/* Call:                                                                               */
/* SSP Interrupt Flag (SSPIF), after each I2C event (when the master address           */
/* write on the I2C bus or if there is data on the I2C bus)                            */
/*                                                                                     */
/*Tasks:                                                                               */
/* Depending on the status there are different actions that can be made : see comments */
/* in source                                                                           */
/* After each action, the SSP Interrupt Flag SSPIF must be cleared again               */
/*                                                                                     */
/* Warning:                                                                            */
/* The status reading or writing is always from the perspective of the master !!       */
/***************************************************************************************/
void interrupt (void)                          // Interruptroutine
{
    ManageI2CBus();
}


/****************** Main program *******************************************************/

/***************************************************************************************/
/* Tasks of the main program:                                                          */
/*   + MCU initialization (using function InitI2CSlave                                 */
/*   + All the logic is performed in ISR routine!!                                     */
/***************************************************************************************/
void main(void)
{
    // MCU initialization
    InitI2CSlave();


    // Endless loop
    while (1)
    {
          if (ucI2CRegistersStatus[1] == _I2C_STATUS_END)
          {
             ucI2CRegisters[2] = ucI2CRegisters[0] + ucI2CRegisters[1];
             ucI2CRegistersStatus[1] = _I2C_STATUS_IDLE;
          }
    }
}