/***************************************************************************************/
/* Demo for driving PIC16Fxx as I2C-Slave according to Application Note AN734          */
/*                                                                                     */
/* Slave                                                                               */
/*                                                                                     */
/* Description: This example is indended to demonstrate several capabilities of I2C bus*/
/*              - master/slave communication                                           */
/*              - timeout in case of communiation failure                              */
/*              - capability of sending unsigned char variables to 32 different        */
/*                internal UC registers                                                */
/*              - variable data lenght                                                 */
/*                                                                                     */
/* Mikrocontroller:       PIC16F887                                                    */
/* Clock:                 internal clock (8.0 MHz)                                     */
/*                                                                                     */
/* Compiler:              mikroC V6.6.2                                                */
/*                                                                                     */
/* Developer:            Mitroi Nicolae Alexandru                                      */
/* Stert of software development: 01. August 2016                                      */
/* Last updated:                  05. August 2016                                      */
/***************************************************************************************/

/******************************************************************************
#definitions section
******************************************************************************/
 #define _I2C_MAX_REGISTERS_COUNT  32 //Holds the max number of internal registers where master can write data
 #define _I2C_CHIP_ADDRESS         63 //Holds the address received from I2C

/******************************************************************************
Global Variables
******************************************************************************/
  unsigned char cI2CAddress;                                   //Holds the address received from I2C
  unsigned char cI2CData;                                      //Holds the last byte received from I2C
  unsigned char ucI2CRegisters[_I2C_MAX_REGISTERS_COUNT];      //Holds the register where the data will be stored
  unsigned char ucI2CRegistersStatus[_I2C_MAX_REGISTERS_COUNT];//Holds the status of the I2C communication per register
  char cI2CCurrentRegisterNumber;                              //Holds the current register number from where the read/write is performed
  char cI2CTotalMsgCount;                                      //Holds the total number of messages that should be send/received
  char cI2CCurrentMsgCount;                                    //Holds the last message count received
  char ucI2CCurrentStatus;                                     //Holds the current I2C status
  char cI2CR_W_Operation;                                      //Holds the current operation requested by master
  


/******************************************************************************
Global Constants
******************************************************************************/
  unsigned const char _I2C_STATUS_IDLE = 0;
  unsigned const char _I2C_STATUS_STARTED = 1;
  unsigned const char _I2C_STATUS_INPROGRESS = 2;
  unsigned const char _I2C_STATUS_ERROR = 3;
  unsigned const char _I2C_STATUS_END = 4;

  unsigned const char _I2C_HEADER_INVALID = -1;
  unsigned const char _I2C_HEADER_VALID = 0;

  unsigned const char _I2C_ADDRESS = 0;
  unsigned const char _I2C_DATA = 1;

  unsigned const char _I2C_WRITE = 0;
  unsigned const char _I2C_READ = 1;
  

/******************************************************************************
Function prototipes
******************************************************************************/
char ParseI2CHeader(char input);
char ParseI2CData(char input);
void ParseI2CInput(char input);
char PrepareResponseData(char input);
void ManageI2CBus();
void InitI2CSlave();